﻿
Public Class RoboTurretDemo

    Dim port As New System.IO.Ports.SerialPort
    Dim bytepacketsend As New Dictionary(Of String, Byte)
    Dim pantilt4095 As New Dictionary(Of String, Integer)

    Dim intCameraPixelWidth As Integer
    Dim intCameraPixelHeight As Integer
    Dim intCenterPan As Integer
    Dim intCenterTilt As Integer
    Dim intOffsetPan As Integer
    Dim intOffsetTilt As Integer
    Dim intTempPan As Integer
    Dim intTempTilt As Integer
    Dim cogX As Integer
    Dim cogY As Integer
    Dim intBlobSizeMinimum As Integer
    Dim intFactor As Integer


#Region "Form Open & Close"

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        'Populate the com port drop down
        For Each port As String In System.IO.Ports.SerialPort.GetPortNames()
            cmbPorts.Items.Add(port)
            cmbPorts.Text = port
        Next



        'sending byte packet
        bytepacketsend.Add("header_byte", CByte(255)) ' always begin packet with 255 header byte - static, leave alone
        bytepacketsend.Add("panH", CByte(8))
        bytepacketsend.Add("panL", CByte(8))
        bytepacketsend.Add("tiltH", CByte(8))
        bytepacketsend.Add("tiltL", CByte(8))
        bytepacketsend.Add("buttons", CByte(0))
        bytepacketsend.Add("ext_instructions", CByte(8))
        bytepacketsend.Add("checksum", CByte((255 - (CInt(bytepacketsend("panL")) + CInt(bytepacketsend("panH")) + CInt(bytepacketsend("tiltL")) + CInt(bytepacketsend("tiltH")) + CInt(bytepacketsend("buttons")) + CInt(bytepacketsend("ext_instructions"))) Mod 256)))

        'we use these to store a 0 to 4095 value from the scrollbars to make life easy when drawing the crosshairs in pbAim_Paint
        pantilt4095.Add("pan", 2048)
        pantilt4095.Add("tilt", 2048)



    End Sub

    Private Sub FormDemo_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ' if the port is still open when the form is closing, be sure to default all values and close the port.
        If port.IsOpen Then
            Try
                ResetDefaults()
                port.Close()
            Catch ex As Exception
                Console.WriteLine("Error updating values: " + ex.Message)
            End Try
        End If
    End Sub

    Sub ResetDefaults()
        bytepacketsend("header_byte") = CByte(255)
        bytepacketsend("panH") = CByte(8)
        bytepacketsend("panL") = CByte(8)
        bytepacketsend("tiltH") = CByte(8)
        bytepacketsend("tiltL") = CByte(8)
        bytepacketsend("buttons") = CByte(0)
        bytepacketsend("ext_instructions") = CByte(8)
        bytepacketsend("checksum") = CByte((255 - (CInt(bytepacketsend("panL")) + CInt(bytepacketsend("panH")) + CInt(bytepacketsend("tiltL")) + CInt(bytepacketsend("tiltH")) + CInt(bytepacketsend("buttons")) + CInt(bytepacketsend("ext_instructions"))) Mod 256))

        pantilt4095("pan") = 2048
        pantilt4095("tilt") = 2048
        hsbPan.Value = 2048
        vsbTilt.Value = 2048
        pbAim.Refresh()
        UpdateValues()
    End Sub


#End Region


#Region "Serial Communication"


    Private Sub UpdateValues()

        txtServoPan.Text = pantilt4095("pan")
        txtServoTilt.Text = pantilt4095("tilt")

        ' Adding limits for tilt when bracket is installed
        If pantilt4095("tilt") < 1024 Then Exit Sub
        If pantilt4095("tilt") > 3072 Then Exit Sub


        'Do our checksum at the last moment to make sure it is up to date
        bytepacketsend("checksum") = CByte((255 - (CInt(bytepacketsend("panH")) _
                                                   + CInt(bytepacketsend("panL")) _
                                                   + CInt(bytepacketsend("tiltH")) _
                                                   + CInt(bytepacketsend("tiltL")) _
                                                   + CInt(bytepacketsend("buttons")) _
                                                   + CInt(bytepacketsend("ext_instructions"))) Mod 256))

        If port.IsOpen Then
            Try
                port.Write(bytepacketsend.Values.ToArray(), 0, bytepacketsend.Count())
            Catch ex As Exception
                Console.WriteLine("Error updating values: " + ex.Message)
            End Try
        End If

    End Sub


    Private Sub btnSetPort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSetPort.Click

        Try
            If Not port.IsOpen Then
                ' Setup SerialPort
                port.PortName = cmbPorts.Text
                port.BaudRate = 38400
                port.DataBits = 8
                port.StopBits = System.IO.Ports.StopBits.One
                port.Parity = System.IO.Ports.Parity.None
                port.Open()
                hsbPan.Enabled = True
                vsbTilt.Enabled = True
                btnSetPort.Text = "Close Port"
                UpdateValues()
            Else
                Call ResetDefaults()
                port.Close()
                hsbPan.Enabled = False
                vsbTilt.Enabled = False
                btnSetPort.Text = "Open Port"
            End If
        Catch ex As Exception
            MessageBox.Show("Communications Error: " + ex.Message)
        End Try

    End Sub



#End Region


#Region "Pan & Tilt Controls"

    Private Sub ScrollBars(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles vsbTilt.Scroll, hsbPan.Scroll
        fncCalculateBytes(4095 - hsbPan.Value, 4095 - vsbTilt.Value)
        pbAim.Refresh()
        UpdateValues()
    End Sub

    Private Sub pbAim_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbAim.MouseMove
        ' only move the servos when the left mouse button is clicked,
        ' the port is open, and the location of the mouse is within the correct intBuffer.
        ' the box is 512 x 512 since 4095 x 4095 would be HUGE on the form.
        If e.Button = MouseButtons.Left AndAlso port.IsOpen Then
            If e.Y > 0 AndAlso e.Y <= 512 AndAlso e.X > 0 AndAlso e.X <= 512 Then
                'we do "(e.X * 2) - 1" because the picturebox is half size at 2048x2048. We need to convert that over to 0 to 4095.
                'we actually are only getting 1 to 4095, but if we can live with that so can you.
                fncCalculateBytes(4095 - ((e.X * 8) - 1), 4095 - ((e.Y * 8) - 1))
                hsbPan.Value = (e.X * 8) - 1
                vsbTilt.Value = (e.Y * 8) - 1
                pbAim.Refresh()
                UpdateValues()
            End If
        End If
    End Sub

    Private Sub fncCalculateBytes(ByVal intPan As Integer, ByVal intTilt As Integer)

        'for the servo horn 4095 is fully rotated left / 0 is fully rotated right
        'thus tilt going up is the # growing toward 4095
        'and panning left (CCW) is the # growing toward 4095

        Dim r As Double
        Dim i As Integer

        'store the 0 - 4095 values
        pantilt4095("pan") = intPan
        pantilt4095("tilt") = intTilt

        'AX & RX Dynamixels have a resolution of 1024 steps (0 to 4095) over 300 degrees
        'But a byte can only store a max # of 255 so we have to do some math magic to split the information into 2 bytes
        i = Math.Truncate(intPan / 256)
        r = intPan - (i * 256)
        bytepacketsend("panL") = CByte(r)
        bytepacketsend("panH") = CByte(i)

        i = Math.Truncate(intTilt / 256)
        r = intTilt - (i * 256)
        bytepacketsend("tiltL") = CByte(r)
        bytepacketsend("tiltH") = CByte(i)

    End Sub

    Private Sub pbAim_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbAim.MouseUp
        pbAim.Refresh()
    End Sub

    Private Sub pbAim_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles pbAim.Paint
        ' create a graphics object on the PictureBox and draw two crosshair lines with it.
        Dim gfx As Graphics = e.Graphics
        Dim myPen As New Pen(Color.Black)

        gfx.DrawLine(myPen, New Point((4095 - pantilt4095("pan")) / 8, 0), New Point((4095 - pantilt4095("pan")) / 8, pbAim.Height)) 'Draw verticle line
        gfx.DrawLine(myPen, New Point(0, (4095 - pantilt4095("tilt")) / 8), New Point(pbAim.Width, (4095 - pantilt4095("tilt")) / 8)) 'Draw horizontal line
    End Sub

#End Region


    Private Sub btnCenter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCenter.Click
        ResetDefaults()
    End Sub
End Class

